
# nmake makefile, creates HDPMI32.EXE.
# tools used:
#  - MASM 6.1 (ml.exe)
#  - MS OMF LIB (lib16.exe)
#  - VALX (valx.exe, version >= 3.7.1.4)

# The current version of the HDPMI binary is a challenge for
# all linkers, because it requires full control over segment ordering.
# That's why most linkers fail now.


!include <..\dirs>

!ifndef DEBUG
DEBUG = 0
!endif

!if $(DEBUG)
ASMD=-Zi -D_DEBUG
LINKD=/LI
!endif

SRCMODS = \
!include modules.inc
OBJNAMES = $(SRCMODS:.ASM=.OBJ)
!if $(DEBUG)
CP = 0
OBJMODS = $(OBJNAMES:.\=STD32D\)
!else
CP = 1
OBJMODS = $(OBJNAMES:.\=STD32\)
!endif

LIB16 = lib16.exe /nologo

!if $(DEBUG)
LINKER = 2
!else
LINKER = 1
!endif

# 1: currently VALX is used, but make sure version is >= 3.7.1.4.
# 2: MS link works , but header size is 200h bytes, that's why SHRMZHDR
#    is called after the link step.
# 3: the digital mars linker cannot be used. it has problems with segment
#    ordering.
# 4: qlink doesn't work, some relocation targets are wrong
# 5: tlink doesn't work

!if $(LINKER)==1
LINK  = valx.exe
LOPTS = /NCI /32 /MAP /DET:2
!elseif $(LINKER)==2
LINK  = link16.exe
LOPTS = /NOLOGO/MAP:FULL/NOI/NOE/NON/ONERROR:NOEXE $(LINKD)
!elseif $(LINKER)==3
LINK  = $(DMCDIR)\link.exe
LOPTS = /NOLOGO/FAR/MAP:FULL/NOREO/NOI/NOE/NON/ONERROR:NOEXE $(LINKD)
!elseif $(LINKER)==4
LINK  = c:\qlink\qlink.exe
LOPTS = /NOLOGO/FAR/MAP:FULL/NOI/NOE/NON/ONERROR:NOEXE $(LINKD)
!else
LINK  = tlink.exe
LOPTS = /3 /Td /s /c
!endif

AS    = ml.exe
AFLAGS= -nologo -c -Cp -Sg -D?32BIT=1 $(ASMD) -I$(INC32DIR) -Fl$* -Fo$*
NAME  = HDPMI32
SRC   = hdpmi

DEPS0 = hdpmi.inc external.inc version.inc

!if $(DEBUG)
OUTSTD=STD32D
OUTNOLFN=NOLFN32D
OUTHSEXT=HSEXT32D
OUTSTUB=STUB32D
OUTIOPL0=IOPL032D
!else
OUTSTD=STD32
OUTNOLFN=NOLFN32
OUTHSEXT=HSEXT32
OUTSTUB=STUB32
OUTIOPL0=IOPL032
!endif

.SUFFIXES: .asm .obj

.asm{$(OUTSTD)}.obj:
    $(AS) $(AFLAGS) $<

# the following versions are created:
# 1. STD32:   standard version with LFN support
# 2. NOLFN32: non-LFN version
# 3. HSEXT32: version with host stack in extended memory [Hiword(ESP) != 0]
#             which requires very little DOS ram, but will not work with some
#             clients (DOS4G!)
# 4. STUB32:  stub version which will be used to create HDLD32.BIN
# 5. IOPL032: version which runs clients with IOPL=0 (Virtual PC)

ALL: $(OUTSTD)\$(NAME).EXE $(OUTNOLFN)\$(NAME).EXE $(OUTHSEXT)\$(NAME).EXE $(OUTSTUB)\$(NAME).INC $(OUTIOPL0)\$(NAME).EXE

$(OBJMODS): $(DEPS0)

# create the standard version

$(OUTSTD)\$(NAME).EXE: $(OUTSTD)\$(SRC).obj $(OUTSTD)\$(NAME).lib $(NAME).mak
    $(LINK) @<<
$(LOPTS) $(OUTSTD)\$(SRC).obj, $*.EXE, $*.MAP, $(OUTSTD)\$(NAME).lib;
<<
!if $(CP)
!if $(LINKER)==2 || $(LINKER)==5
    ..\..\BIN\SHRMZHDR.EXE -d $*.EXE
!endif
    ..\..\BIN\SETMZHDR.EXE $*.EXE
     copy $*.EXE ..\..\BIN\*.*
!ifdef TOOLSDIR
     copy $*.EXE $(TOOLSDIR)\*.*
!endif
!endif     

$(OUTSTD)\$(NAME).lib: $(OBJMODS) $(NAME).mak
	@cd $(OUTSTD)
	@erase $(NAME).lib
	$(LIB16) $(NAME).lib $(OBJNAMES:.\=+);
	$(LIB16) $(NAME).lib -$(SRC).OBJ;
    @cd ..

# create the "no LFN" version

$(OUTNOLFN)\$(NAME).EXE: $(OUTSTD)\$(SRC).obj $(OUTNOLFN)\INT21API.obj $(OUTSTD)\$(NAME).lib $(NAME).mak
    $(LINK) @<<
$(LOPTS) $(OUTSTD)\$(SRC).obj $(OUTNOLFN)\INT21API.obj, $*.EXE, $*.MAP, $(OUTSTD)\$(NAME).lib;
<<
!if $(LINKER)==2 || $(LINKER)==5
    ..\..\BIN\SHRMZHDR.EXE -d $*.EXE
!endif    
    ..\..\BIN\SETMZHDR.EXE $*.EXE

$(OUTNOLFN)\INT21API.obj: INT21API.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?LFNSUPPORT=0 INT21API.asm

# create the "host stack in extended memory" version

HSMODS=$(OUTHSEXT)\$(SRC).obj $(OUTHSEXT)\init.obj $(OUTHSEXT)\switch.obj $(OUTHSEXT)\i31swt.obj

$(OUTHSEXT)\$(NAME).EXE: $(HSMODS) $(OUTSTD)\$(NAME).lib $(NAME).mak
    $(LINK) @<<
$(LOPTS) $(HSMODS), $*.EXE, $*.MAP, $(OUTSTD)\$(NAME).lib;
<<
!if $(LINKER)==2 || $(LINKER)==5
    ..\..\BIN\SHRMZHDR.EXE -d $*.EXE
!endif    
    ..\..\BIN\SETMZHDR.EXE $*.EXE

$(OUTHSEXT)\$(SRC).obj: $(SRC).asm $(DEPS0)
     $(AS) $(AFLAGS) -D?HSINEXTMEM=1 $(SRC).asm

$(OUTHSEXT)\init.obj: init.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?HSINEXTMEM=1 init.asm

$(OUTHSEXT)\switch.obj: switch.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?HSINEXTMEM=1 switch.asm

$(OUTHSEXT)\i31swt.obj: i31swt.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?HSINEXTMEM=1 i31swt.asm

# create the stub version
# EXTRSECT: convert binary to asm include
# -i: create include
# -m: assume MZ binary

$(OUTSTUB)\$(NAME).INC: $*.BIN $(NAME).mak
    ..\..\TOOLS\EXTRSECT.EXE -m -i $*.BIN $*.INC

$(OUTSTUB)\$(NAME).BIN: $(OUTSTUB)\$(SRC).obj $(OUTSTUB)\init.obj $(OUTSTD)\$(NAME).lib $(NAME).mak
    $(LINK) @<<
$(LOPTS) $(OUTSTUB)\$(SRC).obj $(OUTSTUB)\init.obj, $*.BIN, $*.MAP, $(OUTSTD)\$(NAME).lib;
<<

$(OUTSTUB)\$(SRC).obj: $(SRC).asm $(DEPS0)
     $(AS) $(AFLAGS) -D?STUB=1 $(SRC).asm

$(OUTSTUB)\init.obj: init.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?STUB=1 init.asm

# create the IOPL 0 version

$(OUTIOPL0)\$(NAME).EXE: $(OUTIOPL0)\$(SRC).obj $(OUTIOPL0)\switch.obj $(OUTIOPL0)\i31swt.obj $(OUTSTD)\$(NAME).lib $(NAME).mak
    $(LINK) @<<
$(LOPTS) $(OUTIOPL0)\$(SRC).obj $(OUTIOPL0)\switch.obj $(OUTIOPL0)\i31swt.obj, $*.EXE, $*.MAP, $(OUTSTD)\$(NAME).lib;
<<
!if $(LINKER)==2 || $(LINKER)==5
    ..\..\BIN\SHRMZHDR.EXE -d $*.EXE
!endif    
    ..\..\BIN\SETMZHDR.EXE $*.EXE

$(OUTIOPL0)\$(SRC).obj: $(SRC).asm $(DEPS0)
     $(AS) $(AFLAGS) -D?PMIOPL=00 $(SRC).asm

$(OUTIOPL0)\i31swt.obj: i31swt.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?PMIOPL=00 i31swt.asm

$(OUTIOPL0)\switch.obj: switch.asm $(DEPS0)
     $(AS) $(AFLAGS) -D?PMIOPL=00 switch.asm

clean:
    @del $(OUTSTD)\*.obj
    @del $(OUTSTD)\*.lib
    @del $(OUTSTD)\*.exe
    @del $(OUTSTD)\*.lst
    @del $(OUTSTD)\*.map
    @del $(OUTNOLFN)\*.obj
    @del $(OUTNOLFN)\*.exe
    @del $(OUTNOLFN)\*.lst
    @del $(OUTNOLFN)\*.map
    @del $(OUTHSEXT)\*.obj
    @del $(OUTHSEXT)\*.exe
    @del $(OUTHSEXT)\*.lst
    @del $(OUTHSEXT)\*.map
    @del $(OUTSTUB)\*.obj
    @del $(OUTSTUB)\*.bin
    @del $(OUTSTUB)\*.lst
    @del $(OUTSTUB)\*.map
    @del $(OUTSTUB)\*.inc
    @del $(OUTIOPL0)\*.obj
    @del $(OUTIOPL0)\*.exe
    @del $(OUTIOPL0)\*.lst
    @del $(OUTIOPL0)\*.map
